/* prn_ctrl.c        by Eric T. Nielsen    10/90
 *                   A simple utility to send control bytes
 *                   through the LPT1 port to a printer.
 *                   compiles under Borland Turbo C  2.0
 */
#include <stdio.h>
#include <conio.h>
#include <bios.h>

#define SEND 0     /* use biosprint() to send a byte value */
#define LPT1 0     /* the third parameter specifies the port to send to */
#define LPT2 1

#define psend(byt) biosprint(SEND,byt,LPT1)   /* biosprint() is the Turbo C*/
                                              /* access to the BIOS LPT    */
                                              /* port routine.             */
                                              /* _bios_printer() is the    */
                                              /* equivalent routine in     */
                                              /* Microsoft C  5.0          */

void  send_bytes(void),                   /* send bytes to printer    */
      send_num(int num),                  /* send specific # of bytes */
      send_until(void),                   /* send bytes until a,b,c,d */
      test_print(void);                   /* test printer             */



main()
{
   char ch;
   int  running = 1;

   clrscr();                              /* explanation */
   printf("\nPRINTER CONTROL\n");
   printf("\nThis is a simple utility that allows you to send a succession");
   printf("\nof control bytes to your printer through the LPT1 port.");
   printf("\nIt would be of use only to someone possessing a chart of the");
   printf("\ncontrol codes their printer responds to, expressed as");
   printf("\nhexadecimal values.\n");
   printf("\nType lowercase s, t, or x to select options.");
   printf("\nWith [s]end, you can specify the number of bytes you wish");
   printf("\nto send to the printer, or, if you specify a negative");
   printf("\nnumber, you can send any number of bytes, terminated by");
   printf("\nthe values  a,b,c,d  consecutively.");
   printf("\nEnter one- or two-digit hexadecimal values.\n");
   printf("\n[t]est printer  sends a succession of ASCII characters to");
   printf("\nthe printer so you can see the effect of printer commands.\n");


                                          /* main loop */
   do{
      printf("\n[s]end control bytes to printer\n[t]est printer\ne[x]it\n> ");

      ch=getch(); switch(ch){
         case 's': send_bytes();  break;
         case 't': test_print();  break;
         case 'x': running = 0;
      }
   }while(running);
}


void send_bytes(void)                     /* send bytes to printer */
{
   int num;

   printf("\n# of bytes to send: ");
   scanf("%d",&num);

   if(num>0) send_num(num);
   else send_until();
}

void send_num(int num)                  /* send a specific number of bytes */
{
   int i, bytes[40];

   printf("Enter a sequence of %d control bytes, expressed\n",num);
   printf("as 1 or 2-digit hexadecimal values:\n\n");
   for(i=0; i<num; i++)
   {
      printf("%d>  ",i+1);
      scanf("%x",&bytes[i]);
   }

   printf("sending  ");
   for(i=0; i<num; i++)
   {
      psend(bytes[i]);
      printf("%02x  ",bytes[i]);
   }
   printf("to printer\n");
}

void send_until(void)                     /* send bytes until a,b,c,d */
{
   int i, num, bytes[150];

   bytes[0] = 0;
   printf("Enter a sequence of control bytes, expressed as 1 or 2-digit\n");
   printf("hexadecimal values. Terminate sequence by entering a,b,c, and d:\n\n");
   for(i=1; i<150; i++)
   {
      printf("%d> ",i);
      scanf("%x", &bytes[i]);

      if(bytes[i]==0xD)
         if(bytes[i-1]==0xC)
            if(bytes[i-2]==0xB);
               if(bytes[i-3]==0xA) break;

   }
   if(i==150) printf("--max byte sequence length exceeded--\n");
   num = i-4;
   if(num==0) return;

   printf("sending  ");
   for(i=1; i<=num; i++)
   {
      psend(bytes[i]);
      printf("%02x  ",bytes[i]);
   }
   printf("to printer\n");
}


void test_print(void)                     /* test the printer */
{
   int i, j;

   printf("\nFour sets of ASCII 0x21 through 0x7E sent to printer\n");
   for(i=0; i<4; i++)
      for(j=0x21; j<0x7f; j++) psend(j);

   psend(0xD); psend(0xA); psend(0xA);    /* space down two lines */


   /* the following line (commented out) will print the
      positions of ten of the printer's TAB settings:
   for(i=0; i<=10; i++){ psend(0x30 + i); psend(9); }
   */
}